/**
 * Integrate JSHint into Webuilder
 *
 * Nodejs and JSHint have to be installed on system first
 * http://jshint.com/install/
 *
 * @category  WeBuilder Plugin
 * @package   JSHint Integration
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2017
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.1
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     35   function JSHint(Sender)
 *    102   function OnMenuSelect()
 *    111   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 3
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */


/**
 * Process current document through JSHint
 *
 * @param  object   Sender
 *
 * @return void
 */
function JSHint(Sender) {

    if (Sender.DocType != dtJScript) {
        return;
    }

    if (Editor.Modified) {
        if (Script.ReadSetting("Show save dialog?", "0") == "1") {
            if (!Confirm("Document must be saved before running JSHint.\n\nSave document?")) {
                 return;
            }
        }
        Document.Save(Sender.FileName);
    }

	var file = ExtractFileName(Sender.Filename);
	var path = ExtractFilePath(Sender.Filename);

	if ((!FileExists(path + "package.json")) && (!FileExists(path + ".jshintrc"))) {
		// If no config exists in current path, then copy the default config to path
		var fso = CreateOleObject("Scripting.FileSystemObject");
		fso.CopyFile(Script.Path + ".jshintrc", path);

		if (Confirm("Default \".jshintrc\" file have been copied into current path.\nYou can modify this to change the behaviour of JSHint.\n\nDo you want to edit it now?")) {
			Documents.OpenDocument(path + ".jshintrc");
			return;
		}
    }

	var nodejsCmd = RegexReplace(Script.Path, "(\\\\[^\\\\]*){5}$", "", true) +  "\\npm\\jshint.cmd";
	// Checking if nodejs and jshint have been installed
	if (!FileExists(nodejsCmd)) {
		Alert("Error: Nodejs jshint.cmd not found!\nMake sure you have installed Nodejs and the JShint module.");
		return;
	}

	// Set reporter to use custom script to get "WeBuilder ready" result back
    var options = " --reporter=" + Script.Path + "wbreporter.js";

	var jshint = nodejsCmd + " " + options + " \"" + Sender.Filename +"\"", report = "";
   	ExecuteCommand(jshint, report);

    Script.ClearMessages;

    if (report != "") {

		// Loop through report and display each line in Message panel.
	    var SL = new TStringList;
		SL.Text = report;
	    for (var i=0;i<SL.Count;i++) {
        	Script.Message(SL[i]);
		}

		Script.Message("JSHint found " + _t(SL.Count) + " errors in file \"" + file + "\"");

		delete SL;
    }
	else {
		Script.Message("JSHint found no errors in file \"" + file + "\"");
	}
}

/**
 * Function triggered when user select plugin menu item
 *
 * @return void
 */
function OnMenuSelect() {
    JSHint(Document);
}

/**
 * Show info when plugin is installed
 *
 * @return void
 */
function OnInstalled() {
    Alert("JSHint 1.1 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");

var act = Script.RegisterDocumentAction("", "JSHint", "", "OnMenuSelect"), icon = new TBitmap;
LoadFileToBitmap(Script.Path + "jshint_icon.png", icon);
Actions.SetIcon(act, icon);
delete icon;
